import pytest
import os
import sys
import numpy as np

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import gext
import gext.descriptors
import gext.fitting
import gext.grassmann
import utils

SMALL = 1e-10

@pytest.mark.parametrize("datafile", ["urea.json", "glucose.json"])
def test_descriptor_fitting(datafile):

    # load test data from json file
    data = utils.load_json(f"tests/{datafile}")
    nelectrons = data["nelectrons"]
    natoms = data["trajectory"].shape[1]
    nbasis = data["overlaps"].shape[1]
    nframes = data["trajectory"].shape[0]

    # initialize an extrapolator
    extrapolator = gext.Extrapolator(nelectrons, nbasis, natoms, nsteps=nframes)

    # load data in the extrapolator
    for (coords, coeff, overlap) in zip(data["trajectory"],
            data["coefficients"], data["overlaps"]):
        extrapolator.load_data(coords, coeff, overlap)

    # we check if the error goes down with a larger data set
    errors = []
    descriptors = extrapolator.descriptors.get(10)
    target = descriptors[-1]

    for start in range(0, 9):
        vectors = descriptors[start:-1]
        fit_coefficients = gext.fitting.linear(vectors, target)
        fitted_target = gext.fitting.linear_combination(vectors, fit_coefficients)
        errors.append(np.linalg.norm(target - fitted_target, ord=np.inf))

    assert errors[0] < errors[-1]

    # we check that we get a zero error if we put the target in the vectors
    # used for the fitting
    vectors = descriptors[:-1]
    vectors[0] = target
    fit_coefficients = gext.fitting.linear(vectors, target)
    fitted_target = gext.fitting.linear_combination(vectors, fit_coefficients)

    assert np.linalg.norm(target - fitted_target, ord=np.inf) < SMALL
